/* 
 * Label the clusters of bonded spins, using the ants-in-the-labyrinth
 * algorithm.
 * 
 * The array cluster gives the cluster label of a site, and the arrays
 * X_cluster, Y_cluster and Z_cluster give the coordinates of all sites 
 * in the cluster being constructed. 
 */

#include "comext.h"
#include <stdio.h>

label_clusters()
{
int i,j,isweep,n;
int istart,jstart;
int cluster_number,this_cluster,current_size,current_sites,new_sites;
int cluster_label;
int negi,negj,posi,posj;


      for (i=0;i<sites;i++) csize[i] = 0;
      
      /* Initialize the cluster label */
      for (i=0;i<isiz;i++)
         for (j=0;j<jsiz;j++)
	    cluster[i][j] = -1;

      cluster_label = -1;

      /* Label the clusters */
      for (jstart=0;jstart<jsiz;jstart++) {
      for (istart=0;istart<isiz;istart++) {

      if (cluster[istart][jstart] < 0) { /* We have a new cluster to label */

         cluster_label++;
         cluster_number = 0;
         cluster_size = 0;
         new_sites = 1;
         cluster[istart][jstart] = cluster_label;
         X_cluster[cluster_number] = istart;
         Y_cluster[cluster_number] = jstart;

         while (new_sites) { /* Keep labeling them */

	    current_size = cluster_size;
	    cluster_size += new_sites;
	    current_sites = new_sites;
	    new_sites = 0;

	    /* Expand the ant frontier */
            for (n=0;n<current_sites;n++) {

	       this_cluster = current_size + n;
	       i = X_cluster[this_cluster];
	       j = Y_cluster[this_cluster];

               /* positive i direction */
	       posi = plusi(1);
	       if ((cluster[posi][j] < 0) && ibond[i][j]) {
	          new_sites++;
	          cluster_number++;
	          X_cluster[cluster_number] = posi;
	          Y_cluster[cluster_number] = j;
                  cluster[posi][j] = cluster_label;
               }

	       /* negative i direction */
	       negi = minusi(1);
	       if ((cluster[negi][j] < 0) && ibond[negi][j]) {
	          new_sites++;
	          cluster_number++;
	          X_cluster[cluster_number] = negi;
	          Y_cluster[cluster_number] = j;
                  cluster[negi][j] = cluster_label;
               }

	       /* positive j direction */
	       posj = plusj(1);
	       if ((cluster[i][posj] < 0) && jbond[i][j]) {
	          new_sites++;
	          cluster_number++;
	          X_cluster[cluster_number] = i;
	          Y_cluster[cluster_number] = posj;
                  cluster[i][posj] = cluster_label;
               }

	       /* negative j direction */
	       negj = minusj(1);
	       if ((cluster[i][negj] < 0) && jbond[i][negj]) {
	          new_sites++;
	          cluster_number++;
	          X_cluster[cluster_number] = i;
	          Y_cluster[cluster_number] = negj;
                  cluster[i][negj] = cluster_label;
               }
         
	    } /* end of for loop over ant frontier */
         } /* end of while loop over new sites */

         csize[cluster_label] = cluster_size;

      }  /* end of loop testing if cluster has been done */
      }  /* end of loop over i */
      }  /* end of loop over j */

      Nclusters = cluster_label + 1;

#ifdef DEBUG
checklabels(cluster,ibond,jbond);
printcluster(cluster,ibond,jbond,0);
#endif DEBUG

}
